// PointSprite.geom

// SetParameter GL_GEOMETRY_INPUT_TYPE_EXT=GL_TRIANGLES_ADJACENCY_EXT
// SetParameter GL_GEOMETRY_OUTPUT_TYPE_EXT=GL_TRIANGLES
// SetParameter GL_GEOMETRY_VERTICES_OUT_EXT=24

#version 120
#extension GL_EXT_geometry_shader4 : enable
#extension GL_EXT_gpu_shader4 : enable

uniform float shadowLength;

varying in vec3 positionIn[];
varying in vec4 colorIn[];

//varying in float pointSizeIn[];
varying out vec4 color;

vec3 CastedS(const vec3 nlightVec, const vec3 pos){
    const float offsetFactor = 0.001;
    return pos + nlightVec * shadowLength * offsetFactor;
}

vec3 CastedL(const vec3 nlightVec, const vec3 pos){
    return pos + nlightVec * shadowLength;
}

void main(void){

    vec3 lpos = gl_LightSource[0].position.xyz;
    float lw = gl_LightSource[0].position[3];

    vec3 lightVec0 = lpos - positionIn[0] * lw;
    
    vec3 polyNormal = cross(vec3(positionIn[2] - positionIn[0]), vec3(positionIn[4] - positionIn[0]));
    float litOrNot = dot(lightVec0, polyNormal);
    
    if (litOrNot <= 0) return;
    
    vec3 polyNormalABdash = cross(vec3(positionIn[1] - positionIn[0]), vec3(positionIn[2] - positionIn[0]));
    float litOrNotABdash = dot(lightVec0, polyNormalABdash);

    vec3 lightVec2 = lpos - positionIn[2] * lw;
    vec3 polyNormalBCdash = cross(vec3(positionIn[3] - positionIn[2]), vec3(positionIn[4] - positionIn[2]));
    float litOrNotBCdash = dot(lightVec2, polyNormalBCdash);

    vec3 lightVec4 = lpos - positionIn[4] * lw;
    vec3 polyNormalCAdash = cross(vec3(positionIn[5] - positionIn[4]), vec3(positionIn[0] - positionIn[4]));
    float litOrNotCAdash = dot(lightVec4, polyNormalCAdash);

    //vec3 offsetS = nlightVec * shadowLength * 0.001;
    //vec3 offsetL = nlightVec * shadowLength;

    color = colorIn[0];

    mat4 mat = gl_ProjectionMatrix;

    vec3 ltVec0 = lpos - positionIn[0] * lw;
    vec3 ltVec2 = lpos - positionIn[2] * lw;
    vec3 ltVec3 = lpos - positionIn[4] * lw;
    vec3 nltVec0 = -normalize(lightVec0);
    vec3 nltVec2 = -normalize(lightVec2);
    vec3 nltVec4 = -normalize(lightVec4);

    gl_Position = mat * vec4(CastedS(nltVec0, positionIn[0]), 1);
    EmitVertex();
    gl_Position = mat * vec4(CastedS(nltVec2, positionIn[2]), 1);
    EmitVertex();
    gl_Position = mat * vec4(CastedS(nltVec4, positionIn[4]), 1);
    EmitVertex();
    EndPrimitive();

    if (litOrNotABdash <= 0){
	gl_Position = mat * vec4(CastedS(nltVec0, positionIn[0]), 1);
	EmitVertex();
	gl_Position = mat * vec4(CastedL(nltVec2, positionIn[2]), 1);
	EmitVertex();
	gl_Position = mat * vec4(CastedS(nltVec2, positionIn[2]), 1);
	EmitVertex();
	EndPrimitive();
	
	gl_Position = mat * vec4(CastedS(nltVec0, positionIn[0]), 1);
	EmitVertex();
	gl_Position = mat * vec4(CastedL(nltVec0, positionIn[0]), 1);
	EmitVertex();
	gl_Position = mat * vec4(CastedL(nltVec2, positionIn[2]), 1);
	EmitVertex();
	EndPrimitive();
    }

    if (litOrNotBCdash <= 0){
	gl_Position = mat * vec4(CastedS(nltVec2, positionIn[2]), 1);
	EmitVertex();
	gl_Position = mat * vec4(CastedL(nltVec4, positionIn[4]), 1);
	EmitVertex();
	gl_Position = mat * vec4(CastedS(nltVec4, positionIn[4]), 1);
	EmitVertex();
	EndPrimitive();

	gl_Position = mat * vec4(CastedS(nltVec2, positionIn[2]), 1);
	EmitVertex();
	gl_Position = mat * vec4(CastedL(nltVec2, positionIn[2]), 1);
	EmitVertex();
	gl_Position = mat * vec4(CastedL(nltVec4, positionIn[4]), 1);
	EmitVertex();
	EndPrimitive();
    }

    if (litOrNotCAdash <= 0){
	gl_Position = mat * vec4(CastedS(nltVec4, positionIn[4]), 1);
	EmitVertex();
	gl_Position = mat * vec4(CastedL(nltVec0, positionIn[0]), 1);
	EmitVertex();
	gl_Position = mat * vec4(CastedS(nltVec0, positionIn[0]), 1);
	EmitVertex();
	EndPrimitive();
	
	gl_Position = mat * vec4(CastedS(nltVec4, positionIn[4]), 1);
	EmitVertex();
	gl_Position = mat * vec4(CastedL(nltVec4, positionIn[4]), 1);
	EmitVertex();
	gl_Position = mat * vec4(CastedL(nltVec0, positionIn[0]), 1);
	EmitVertex();
	EndPrimitive();
    }

    gl_Position = mat * vec4(CastedL(nltVec0, positionIn[0]), 1);
    EmitVertex();
    gl_Position = mat * vec4(CastedL(nltVec4, positionIn[4]), 1);
    EmitVertex();
    gl_Position = mat * vec4(CastedL(nltVec2, positionIn[2]), 1);
    EmitVertex();
    EndPrimitive();
}
